function updateModel(this, modelName, varargin)
% UPDATEMODEL(THIS, MODELNAME) updates model MODELNAME with changes in imported
% AUTOSAR description defined in arxml.importer object THIS. The update/merge
% generates a report detailing updates automatically applied to the model and
% required changes that were not made automatically. The imported description
% must contain the AUTOSAR software component mapped by the model.
% 
% Example:
% Update model 'mySWC.slx' with AUTOSAR arxml changes described in
% 'updatedSWC.arxml'
% 
% open_system('mySWC');
% obj = arxml.importer('updatedSWC.arxml');
% obj.updateModel('mySWC');

% The update report launches automatically if no value is given to the
% LaunchReport parameter (default is 'on'). To not launch the report, the LaunchReport
% parameter has to be set to 'off'. Example: 
%
% obj.updateModel('mySWC', 'LaunchReport', 'off');
%

%   Copyright 2013 The MathWorks, Inc.

% Check function arguments
argParser = inputParser;
argParser.addRequired('ModelName', @ischar);
argParser.addParameter('LaunchReport', 'on', @(x) any(validatestring(x,{'on','off'})));
argParser.parse(modelName, varargin{:});

% Strip out any extension that user may have passed in
[~, modelName] = fileparts(modelName);

launchReport = argParser.Results.LaunchReport;
p_updateModel(this, modelName, 'LaunchReport', launchReport);

% LocalWords:  MODELNAME
